@@ -769,6 +769,133 @@ int32_t SPVM__Net__SSLeay__set_session(SPVM_ENV* env, SPVM_VALUE* stack) {
   return 0;
 }
 
+int32_t SPVM__Net__SSLeay__get_certificate(SPVM_ENV* env, SPVM_VALUE* stack) {
+  
+  int32_t error_id = 0;
+  
+  void* obj_self = stack[0].oval;
+  
+  SSL* ssl = env->get_pointer(env, stack, obj_self);
+  
+  X509* x509 = SSL_get_certificate(ssl);
+  
+  void* obj_x509 = NULL;
+  
+  if (x509) {
+    env->new_pointer_object_by_name(env, stack, "Net::SSLeay::X509", x509, &error_id, __func__, FILE_NAME, __LINE__);
+    if (error_id) { return error_id; }
+    
+    env->set_no_free(env, stack, x509, 1);
+  }
+  
+  stack[0].oval = obj_x509;
+  
+  return 0;
+}
+
+int32_t SPVM__Net__SSLeay__get0_next_proto_negotiated(SPVM_ENV* env, SPVM_VALUE* stack) {
+  
+  int32_t error_id = 0;
+  
+  void* obj_self = stack[0].oval;
+  
+  void* obj_data_ref = stack[1].oval;
+  
+  int32_t* len_ref = stack[2].iref;
+  
+  SSL* ssl = env->get_pointer(env, stack, obj_self);
+  
+  if (!obj_data_ref) {
+    return env->die(env, stack, "The data reference $data_ref must be defined.", __func__, FILE_NAME, __LINE__);
+  }
+  
+  int32_t data_ref_length = env->length(env, stack, obj_data_ref);
+  
+  if (!(data_ref_length == 1)) {
+    return env->die(env, stack, "The length of the data reference $data_ref must be 1.", __func__, FILE_NAME, __LINE__);
+  }
+  
+  const unsigned char* data_ref_tmp[1] = {0};
+  unsigned len_ref_tmp = -1;
+  SSL_get0_next_proto_negotiated(ssl, data_ref_tmp, &len_ref_tmp);
+  
+  if (data_ref_tmp) {
+    void* obj_data = env->new_string_nolen(env, stack, data_ref_tmp[0]);
+    
+    env->set_elem_object(env, stack, obj_data_ref, 0, obj_data);
+  }
+  
+  *len_ref = len_ref_tmp;
+  
+  return 0;
+}
+
+int32_t SPVM__Net__SSLeay__get0_alpn_selected(SPVM_ENV* env, SPVM_VALUE* stack) {
+  
+  int32_t error_id = 0;
+  
+  void* obj_self = stack[0].oval;
+  
+  void* obj_data_ref = stack[1].oval;
+  
+  int32_t* len_ref = stack[2].iref;
+  
+  SSL* ssl = env->get_pointer(env, stack, obj_self);
+  
+  if (!obj_data_ref) {
+    return env->die(env, stack, "The data reference $data_ref must be defined.", __func__, FILE_NAME, __LINE__);
+  }
+  
+  int32_t data_ref_length = env->length(env, stack, obj_data_ref);
+  
+  if (!(data_ref_length == 1)) {
+    return env->die(env, stack, "The length of the data reference $data_ref must be 1.", __func__, FILE_NAME, __LINE__);
+  }
+  
+  const unsigned char* data_ref_tmp[1] = {0};
+  unsigned int len_ref_tmp = -1;
+  SSL_get0_alpn_selected(ssl, data_ref_tmp, &len_ref_tmp);
+  
+  if (data_ref_tmp) {
+    void* obj_data = env->new_string_nolen(env, stack, data_ref_tmp[0]);
+    
+    env->set_elem_object(env, stack, obj_data_ref, 0, obj_data);
+  }
+  
+  *len_ref = len_ref_tmp;
+  
+  return 0;
+}
+
+int32_t SPVM__Net__SSLeay__get_peer_cert_chain(SPVM_ENV* env, SPVM_VALUE* stack) {
+  
+  int32_t error_id = 0;
+  
+  void* obj_self = stack[0].oval;
+  
+  SSL* ssl = env->get_pointer(env, stack, obj_self);
+  
+  STACK_OF(X509)* stack_of_x509 = SSL_get_peer_cert_chain(ssl);
+  
+  void* obj_x509s = NULL;
+  
+  if (stack_of_x509) {
+    int32_t length = sk_X509_num(stack_of_x509);
+    obj_x509s = env->new_object_array_by_name(env, stack, "Net::SSLeay::X509", length, &error_id, __func__, FILE_NAME, __LINE__);
+    
+    for (int32_t i = 0; i < length; i++) {
+      X509* x509 = sk_X509_value(stack_of_x509, i);
+      X509_up_ref(x509);
+      void* obj_x509 = env->new_pointer_object_by_name(env, stack, "Net::SSLeay::X509", x509, &error_id, __func__, FILE_NAME, __LINE__);
+      env->set_elem_object(env, stack, obj_x509s, i, obj_x509);
+    }
+  }
+  
+  stack[0].oval = obj_x509s;
+  
+  return 0;
+}
+
 int32_t SPVM__Net__SSLeay__DESTROY(SPVM_ENV* env, SPVM_VALUE* stack) {
   
   int32_t error_id = 0;
